//*********************************************************************************//
//                                                                                 //
//*********************************************************************************//

#define REMOTE 		_RB7 //_LATB7 //_RB7
#define REMOTE_DIR 	_TRISB7

//=====================================================================
//	CONSTANTS
//=====================================================================
#define F_OSC			4000000		// 4MHz
#define INT_PERIOD		200			// ca. 200s at 4.00 MHz
#define GLOB_C_INIT		5			// Global_Counter init value

#define INTCON_INIT		0b10110000	// intcon register init value
#define OPTION_INIT		0b10001111	// no RBPU, INTEDG falling, slow watchdog
#define PIE1_INIT	   	0b00000000

//====================== Ports ========================================

#define TRISA_INIT	0b00000000	// data direction port A init
#define PORTA_INIT	0b00110000	// port A init value
#define TRISB_INIT	0b00001111	// B0 TV_RC, 1 2 3 prox sens ON, 4 right, 5 left
#define PORTB_INIT	0b01000001	// 1 2 3 prox sens ON,
#define TRISC_INIT	0b10010000	// rx as input
#define PORTC_INIT	0b00000000	// C2=0 mot stop
#define TRISD_INIT	0b00000000	//
#define PORTD_INIT	0b00000000	//
#define TRISE_INIT	0b00000000	//
#define PORTE_INIT	0b00000000	//

// ports at sleep or low power
#define TRISA_NO_IR	0b00101111	// A5 as input -> no power to IR receiver
#define PORTA_SLEEP	0b00011111	// LED off, IR receiver off
//#define PORTB_SLEEP	0b00111111	// Motors off
//#define TRISB_SLEEP	0b01111111	// Motors off

//---------------------- PORTA --------------------------------------------
//	Pins of Motors on Porta
#define Forward1		2
#define Forward2		1

//	Bits of MotorFlags0
#define MOTOR_OFF_ON	MotorFlags0, 0	// Robot is on a point
#define MOTOR_DIR		MotorFlags0, 1	// Movement direction
//#define free	MotorFlags0, 2	//
//#define free	MotorFlags0, 3	//
#define MR_OFF_ON		MotorFlags0, 4	// 1 if motor is turned on
#define ML_OFF_ON		MotorFlags0, 5	//
#define MR_DIR			MotorFlags0, 6	// motor direction
#define ML_DIR			MotorFlags0, 7	// 1 forward

#define MOTOR_INIT		0b11110011	// Motor ON flags
#define FORWARD_LEFT	0b11010011	// 1:
#define FORWARD			0b11110011	// 2:
#define FORWARD_RIGHT	0b11100011	// 3:
#define MOTOR_STOP		0b11000010	// 5:Motor STOP flags
#define BACKWARD_LEFT	0b00010001	// 7
#define BACKWARD		0b00110001	// 8
#define BACKWARD_RIGHT	0b00100001	// 9


//---------------------- Communication IR, RC5 ----------------------------
#define  NR_PERIODS		9	// period of a normal bit (*200us)
#define  NR_BITS		14	// nr of bit to receive or send

//	bits of CommFlags0
#define IR_TV_RECEIVING	CommFlags0, 0	//
#define IR_TV_RECEIVED	CommFlags0, 1	// command received from IR
#define IR_TV_ST_BIT1	CommFlags0, 2	// receiving start bit
//#define STANDBY			CommFlags0, 3	// robot is in stop situation

//---------------------- BehaviorFlags --------------------------------
//	bits of BehaviorFlags
#define DONT_CONTROL	BehaviorFlags, 0	// don't avoid obstacles
#define FOLLOW_RIGHT	BehaviorFlags, 2	// Follow right wall
#define FOLLOW_LEFT		BehaviorFlags, 3	// Follow left wall
#define BEHAVE_AVOID	0b00000000			// constants to modify all the behavior flags
#define BEHAVE_RIGHT	0b00000100			//
#define BEHAVE_LEFT		0b00001000			//

//-----NRF-----//
#define PAYLOAD_SIZE 31
#define CHANGE_STATE 0x0
#define CHANGE_RF 0x1


//=====================================================================
//	LOW LEVEL ROUTINE DECLARATION
//=====================================================================
// predefined "behavior"

// LED activation
#define LightON()  \
output_low(PIN_A4);
#define LightOFF()  \
output_high(PIN_A4);

// TIME functions
#define GetTime() Now
#define GetSeconds() (Now/    1000)
#define GetMinutes() (Now/   60000)
#define GetHours()   (Now/ 3600000)
#define GetDays()    (Now/86400000)

// ---- other not in standard includes ...

//#define W 	 		0
//#define F  			1
#define PORTA  5
#define PORTB  6
#define PORTC  7
#define PORTD  8
#define PORTE  9
#define INTCON  	11
#define ADRESH		0x1E
#define ADRESL		0x9E
#define ADCON0    0x1F
#define TMR0		1
#define PCL			2
#define STATUS 	3
//----- STATUS Bits ----
#define IRP 		7
#define RP1 		6
#define RP0    	5
#define NOT_TO   	4
#define NOT_PD   	3
#define Z        	2
#define DC       	1
#define CARRY    	0
//----- ADCON0 Bits ----
#define ADCS1		7
#define ADCS0 		6
#define CHS2 		5
#define CHS1 		4
#define CHS0 		3
#define GO 			2
#define NOT_DONE  2
#define GO_DONE 	2
#define ADON 		0
//----- INTCON Bits ----
#define GIE 		7
#define PEIE 		6
#define T0IE 		5
#define INTE 		4
#define RBIE 		3
#define T0IF 		2
#define INTF 		1
#define RBIF 		0


