//*********************************************************************************//
//                                                                                 //
//                   Newpuck main program						                   //
//                                                                                 //
//                                                                                 //
//*********************************************************************************//

#define USE_AND_OR 						// To enable AND_OR mask setting during peripheral configuration

#include <p24FJ64GA104.h>
#include <spi.h>
#include <uart.h>
#include <outcompare.h>
#include <timer.h>
#include <ports.h>
#include <PPS.h>
#include <adc.h>
#include "variables.h"
#include "nRF_API.h"
#include "e_remote_control.h"
#include "e_agenda.h"
#include <stdio.h>
#include <math.h>

_CONFIG3(SOSCSEL_IO);								// secondary oscillator off (use RA4 and RB4 as digital IO)
_CONFIG2(FNOSC_FRCDIV & IOL1WAY_OFF & OSCIOFNC_ON);	// fast internal RC oscillator (8 MHz) with configurable postscaler; PPS can be re-configured more than once in runtime; RA3 digital I/O
_CONFIG1(FWDTEN_OFF & JTAGEN_OFF);					// watchdog timer and jtag are disabled

extern unsigned char data_ir;
extern unsigned char command_received;	// IR remote command received flag

void init_rf_module(unsigned char RF_channel) {

	// Set PWR_DOWN bit
	_LATC6 = 0;		// high to low transition of CSN pin
	toSend[0]=NRF_WRITE_REG + NRF_CONFIG;
	toSend[1]=0x0D;
	WriteSPI1(toSend[0]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[1]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	_LATC6 = 1;
	while(DataRdySPI1()) {
    	spi_data = ReadSPI1();
	}

//	_LATC6 = 0;		// high to low transition of CSN pin
//	toSend[0]=NRF_READ_REG + NRF_CONFIG;
//	toSend[1]=0x00;
//	WriteSPI1(toSend[0]);
//	while(!DataRdySPI1());
//	spi_data = ReadSPI1();
//	WriteSPI1(toSend[1]);
//	while(!DataRdySPI1());
//	spi_data = ReadSPI1();
//	printf("config reg:%x\n", spi_data);
//	_LATC6 = 1;
//	while(DataRdySPI1()) {
//    	spi_data = ReadSPI1();
//	}

	// Address width (3 bytes)
	_LATC6 = 0;
	toSend[0]=NRF_WRITE_REG + NRF_SETUP_AW;
	toSend[1]=0x01;
	WriteSPI1(toSend[0]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[1]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	_LATC6 = 1;
	while(DataRdySPI1()) {
    	spi_data = ReadSPI1();
	}

//	_LATC6 = 0;		// high to low transition of CSN pin
//	toSend[0]=NRF_READ_REG + NRF_SETUP_AW;
//	toSend[1]=0x00;
//	WriteSPI1(toSend[0]);
//	while(!DataRdySPI1());
//	spi_data = ReadSPI1();
//	WriteSPI1(toSend[1]);
//	while(!DataRdySPI1());
//	spi_data = ReadSPI1();
//	printf("setup aw reg:%x\n", spi_data);
//	_LATC6 = 1;
//	while(DataRdySPI1()) {
//    	spi_data = ReadSPI1();
//	}

	// Writes TX_Address to nRF24L01
	_LATC6 = 0;
	toSend[0]=NRF_WRITE_REG + NRF_TX_ADDR;
	toSend[1]=(RF_ADDR>>8)&0xFF;
	toSend[2]=RF_ADDR & 0xFF;
	toSend[3]=0x00;
	WriteSPI1(toSend[0]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[1]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[2]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[3]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	_LATC6 = 1;
	while(DataRdySPI1()) {
    	spi_data = ReadSPI1();
	}

	// RX_Addr0 same as TX_Adr for Auto.Ack
	_LATC6 = 0;
	toSend[0]=NRF_WRITE_REG + NRF_RX_ADDR_P0;
	toSend[1]=(RF_ADDR>>8)&0xFF;
	toSend[2]=RF_ADDR & 0xFF;
	toSend[3]=0x0;
	WriteSPI1(toSend[0]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[1]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[2]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[3]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	_LATC6 = 1;
	while(DataRdySPI1()) {
    	spi_data = ReadSPI1();
	}

	// Enable Auto.Ack:Pipe0
	_LATC6 = 0;
	toSend[0]=NRF_WRITE_REG + NRF_EN_AA;
	toSend[1]=0x01;
	WriteSPI1(toSend[0]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[1]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	_LATC6 = 1;
	while(DataRdySPI1()) {
    	spi_data = ReadSPI1();
	}

	// Enable Pipe0
	_LATC6 = 0;
	toSend[0]=NRF_WRITE_REG + NRF_EN_RXADDR;
	toSend[1]=0x01;
	WriteSPI1(toSend[0]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[1]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	_LATC6 = 1;
	while(DataRdySPI1()) {
    	spi_data = ReadSPI1();
	}

	// 500s (+ 86s on-air), 2 re-transmissions
	_LATC6 = 0;
	toSend[0]=NRF_WRITE_REG + NRF_SETUP_RETR;
	toSend[1]=0x12;
	WriteSPI1(toSend[0]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[1]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	_LATC6 = 1;
	while(DataRdySPI1()) {
    	spi_data = ReadSPI1();
	}

	// Select RF channel 40
	_LATC6 = 0;
	toSend[0]=NRF_WRITE_REG + NRF_RF_CH;
	toSend[1]=RF_channel;
	WriteSPI1(toSend[0]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[1]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	_LATC6 = 1;
	while(DataRdySPI1()) {
    	spi_data = ReadSPI1();
	}

	// TX_PWR:0dBm, Datarate:2Mbps, LNA:HCURR
	_LATC6 = 0;
	toSend[0]=NRF_WRITE_REG + NRF_RF_SETUP;
	toSend[1]=0x0f;
	WriteSPI1(toSend[0]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[1]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	_LATC6 = 1;
	while(DataRdySPI1()) {
    	spi_data = ReadSPI1();
	}

	// RX payload size; it isn't needed because the dynamic payload length is activated for ACK+PAYLOAD feature
	_LATC6 = 0;
	toSend[0]=NRF_WRITE_REG + NRF_RX_PW_P0;
	toSend[1]=PAYLOAD_SIZE;
	WriteSPI1(toSend[0]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[1]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	_LATC6 = 1;
	while(DataRdySPI1()) {
    	spi_data = ReadSPI1();
	}

	// enable extra features
	_LATC6 = 0;
	toSend[0]=NRF_ACTIVATE;
	toSend[1]=0x73;
	WriteSPI1(toSend[0]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[1]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	_LATC6 = 1;
	while(DataRdySPI1()) {
    	spi_data = ReadSPI1();
	}

	// enable dynamic payload for pipe0
	_LATC6 = 0;
	toSend[0]=NRF_WRITE_REG + NRF_DYNPD;
	toSend[1]=0x01;
	WriteSPI1(toSend[0]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[1]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	_LATC6 = 1;
	while(DataRdySPI1()) {
    	spi_data = ReadSPI1();
	}			

//	_LATC6 = 0;		// high to low transition of CSN pin
//	toSend[0]=NRF_READ_REG + NRF_DYNPD;
//	toSend[1]=0x00;
//	WriteSPI1(toSend[0]);
//	while(!DataRdySPI1());
//	spi_data = ReadSPI1();
//	WriteSPI1(toSend[1]);
//	while(!DataRdySPI1());
//	spi_data = ReadSPI1();
//	printf("dynpd reg:%x\r\n", spi_data);
//	_LATC6 = 1;
//	while(DataRdySPI1()) {
//    	spi_data = ReadSPI1();
//	}

	// enable payload with ACK and dynamic payload length
	_LATC6 = 0;
	toSend[0]=NRF_WRITE_REG + NRF_FEATURE;
	toSend[1]=0x06;
	WriteSPI1(toSend[0]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[1]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	_LATC6 = 1;
	while(DataRdySPI1()) {
    	spi_data = ReadSPI1();
	}

//	_LATC6 = 0;		// high to low transition of CSN pin
//	toSend[0]=NRF_READ_REG + NRF_FEATURE;
//	toSend[1]=0x00;
//	WriteSPI1(toSend[0]);
//	while(!DataRdySPI1());
//	spi_data = ReadSPI1();
//	WriteSPI1(toSend[1]);
//	while(!DataRdySPI1());
//	spi_data = ReadSPI1();
//	printf("feature reg:%x\r\n", spi_data);
//	_LATC6 = 1;
//	while(DataRdySPI1()) {
//    	spi_data = ReadSPI1();
//	}

	// Set PWR_UP bit, enable CRC(2 bytes) & PRX. MAX_RT & TX_DS enabled
	_LATC6 = 0;
	toSend[0]=NRF_WRITE_REG + NRF_CONFIG;
	toSend[1]=0x0b;
	WriteSPI1(toSend[0]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[1]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	_LATC6 = 1;
	while(DataRdySPI1()) {
    	spi_data = ReadSPI1();
	}

}

void init() {
	unsigned int SPICON1Value;	// Holds the information about SPI configuartion
	unsigned int SPICON2Value;	// Holds the information about SPI configuartion
	unsigned int SPISTATValue;	// Holds the information about SPI Enable/Disable
	unsigned int U1MODEValue;	// Holds the information about UART configuration
	unsigned int U1STAValue;	// Holds the information about UART Enable/Disable
	unsigned int config1_pwm, config2_pwm;	// OCxCON1, OCxCON2
	unsigned long value1_pwm, value2_pwm;	// OCxRS (duty cycle) and OCxR (read-only in pwm mode) 
	unsigned int config1,config2,config3,configport,configscan;

	// clock configuration
	_RCDIV = 0b001;	// divide by 2 => 4 MHz clock


	// PPS (peripheral pin select) configuration
	// ...you can also use the functions: iPPSInput, iPPSOutput, PPSInput, PPSOutput, PPSLock, PPSUnLock
	// unlock registers
//	asm volatile (	"MOV #OSCCON, w1\n"
//					"MOV #0x46, w2\n"
//					"MOV #0x57, w3\n"
//					"MOV.b w2, [w1]\n"
//					"MOV.b w3, [w1]\n"
//					"BCLR OSCCON,#6");


	// Configure Input Functions (Table 9-1)
	iPPSInput(IN_FN_PPS_U1RX, IN_PIN_PPS_RP8);			// assign Uart1 RX to pin RP8
	iPPSInput(IN_FN_PPS_SDI1, IN_PIN_PPS_RP24);			// assign SPI1 Data Input to pin RP24


	// Configure Output Functions (Table 9-2)		
	iPPSOutput(OUT_PIN_PPS_RP9,OUT_FN_PPS_U1TX);		// assign Uart1 TX to pin RP9
	_TRISC6 = 0;										// RC6 as output (SPI1 Slave Select Output)	
	iPPSOutput(OUT_PIN_PPS_RP23, OUT_FN_PPS_SCK1OUT);	// assign SPI1 Clock Output to pin RP23
	iPPSOutput(OUT_PIN_PPS_RP25, OUT_FN_PPS_SDO1);		// assign SPI1 Data Output to pin RP25
	_RB10 = 0;											// motor right backward in known state (0)
	_TRISB10 = 1;										// motor right backward in known state (input)
	iPPSOutput(OUT_PIN_PPS_RP14, OUT_FN_PPS_OC2);		// assign Output Compare 2 (motor right forward) to pin RP14	
	_RB11 = 0;											// motor left backward in known state (0)
	_TRISB11 = 1;										// motor left backward in known state (input)
	iPPSOutput(OUT_PIN_PPS_RP15, OUT_FN_PPS_OC1);		// assign Output Compare 1 (motor left forward) to pin RP15
	iPPSOutput(OUT_PIN_PPS_RP21, OUT_FN_PPS_OC3);		// assign Output Compare 3 (led blue) to pin RP21
	iPPSOutput(OUT_PIN_PPS_RP20, OUT_FN_PPS_OC4);		// assign Output Compare 4 (led green) to pin RP20
	iPPSOutput(OUT_PIN_PPS_RP19, OUT_FN_PPS_OC5);		// assign Output Compare 5 (led red) to pin RP19


//	// lock registers
//	asm volatile (	"MOV #OSCCON, w1\n"
//					"MOV #0x46, w2\n"
//					"MOV #0x57, w3\n"
//					"MOV.b w2, [w1]\n"
//					"MOV.b w3, [w1]\n"
//					"BSET OSCCON, #6" );


	// ports configuration
	_TRISB7 = 1;	// remote control
	
	_TRISA9 = 0;	// sleep
	_RA9 = 1;
	
	_TRISB4 = 1;	// selector
	_TRISA8 = 1;
	_TRISA3 = 1;
	_TRISA2 = 1;

	_TRISA10 = 1;	// motor right direction (not used, set as input)
	_RA10 = 0;
	_TRISA7 = 1;	// motor left direction (not used, set as input)
	_RA7 = 0;	

	LED_IR1_DIR = 0;	// led IR1
	LED_IR1 = 1;
	LED_IR2_DIR = 0;	// led IR2
	LED_IR2 = 1;

	// uart configuration
	CloseUART1();	//dsiable UART if enabled previously
    ConfigIntUART1(UART_RX_INT_EN | UART_RX_INT_PR6 | UART_TX_INT_EN | UART_TX_INT_PR6);	// Enable RX interrupt only
	U1MODEValue = UART_EN | UART_NO_PAR_8BIT | UART_1STOPBIT | UART_MODE_SIMPLEX | UART_BRGH_FOUR;			// 8bit,no parity and 1 stopbit
	U1STAValue = UART_TX_ENABLE;
	// UxBRG (with BRGH=0) = ((FCY/Desired Baud Rate)/16)  1
	// in our case: ((2000000/115200)/16) - 1 = 0
	// the resulting baud rate is: 2000000/(16 (0 + 1)) = 125000 baud
	// the resulting error is: (Calculated Baud Rate  Desired Baud Rate)/Desired Baud Rate = 8%
	// in our case: ((2000000/9600)/16) - 1 = 12
	// the resulting baud rate is: 2000000/(16 (12 + 1)) = 9615 baud
	// the resulting error is: (Calculated Baud Rate  Desired Baud Rate)/Desired Baud Rate = 0.1%
	// in our case: ((2000000/57600)/16) - 1 = 1
	// the resulting baud rate is: 2000000/(16 (1 + 1)) = 62500 baud
	// the resulting error is: (Calculated Baud Rate  Desired Baud Rate)/Desired Baud Rate = 8%
	// in our case: ((2000000/38400)/16) - 1 = 2
	// the resulting baud rate is: 2000000/(16 (2 + 1)) = 41666 baud
	// the resulting error is: (Calculated Baud Rate  Desired Baud Rate)/Desired Baud Rate = 8%
	// in our case: ((2000000/19200)/16) - 1 = 5
	// the resulting baud rate is: 2000000/(16 (5 + 1)) = 20833 baud
	// the resulting error is: (Calculated Baud Rate  Desired Baud Rate)/Desired Baud Rate = 8%
	// UxBRG (with BRGH=1) = ((FCY/Desired Baud Rate)/4)  1
	// in our case: ((2000000/19200)/4) - 1 = 25
	// the resulting baud rate is: 2000000/(4 (25 + 1)) = 19230 baud
	// the resulting error is: (Calculated Baud Rate  Desired Baud Rate)/Desired Baud Rate = 0.1%
	// in our case: ((2000000/57600)/4) - 1 = 7
	// the resulting baud rate is: 2000000/(4 (7 + 1)) = 62500 baud
	// the resulting error is: (Calculated Baud Rate  Desired Baud Rate)/Desired Baud Rate = 8%
	// in our case: ((2000000/38400)/4) - 1 = 12
	// the resulting baud rate is: 2000000/(4 (12 + 1)) = 38461 baud
	// the resulting error is: (Calculated Baud Rate  Desired Baud Rate)/Desired Baud Rate = 0.1%
    OpenUART1(U1MODEValue, U1STAValue, 12); 	//configure UART and enable it at 38400 baud


	// SPI configuration
	CloseSPI1();					// Disbale SPI1 mdolue if enabled previously
    ConfigIntSPI1(SPI_INT_DIS);		// Disable SPI1 interrupt
	// Configure SPI module to transmit 8 bit value in master mode
	SPICON1Value =	ENABLE_SCK_PIN | ENABLE_SDO_PIN | SPI_MODE16_OFF |
                    SPI_SMP_OFF | SPI_CKE_OFF |
                    SLAVE_ENABLE_OFF |   //SPI2CONbits.SSEN
                    CLK_POL_ACTIVE_HIGH |
                    MASTER_ENABLE_ON |
                    SEC_PRESCAL_2_1 |
                    PRI_PRESCAL_1_1;
	SPICON2Value =	FRAME_ENABLE_OFF | FRAME_SYNC_OUTPUT;
    SPISTATValue  = SPI_ENABLE | SPI_IDLE_CON | SPI_RX_OVFLOW_CLR;
    OpenSPI1(SPICON1Value,SPICON2Value,SPISTATValue);


	// pwm configuration
	CloseOC1();					// Disable output compare modules if enabled previously
	CloseOC2();
	CloseOC3();
	CloseOC4();
	CloseOC5();

	ConfigIntOC1(OC_INT_OFF);	// Disable interrupts
	ConfigIntOC2(OC_INT_OFF);
	ConfigIntOC3(OC_INT_OFF);
	ConfigIntOC4(OC_INT_OFF);
	ConfigIntOC5(OC_INT_OFF);

	// left motor
	OC1R = 0;					// initial duty cycle (motor stopped)
	OC1RS = PERIOD_MOTORS;		// period, see timer 2 (PR2) configuration
	OC1CON2bits.SYNCSEL = 0x1F;
	OC1CON2bits.OCTRIG = 0x0;
	OC1CON1bits.OCTSEL = 0x0;	// timer 2 as source
	OC1CON1bits.OCM = 0b110;	// Edge-Aligned PWM mode on OCx

	// right motor
	OC2R = 0;					// initial duty cycle (motor stopped)
	OC2RS = PERIOD_MOTORS;		// period, see timer 5 (PR5) configuration
	OC2CON2bits.SYNCSEL = 0x1F;
	OC2CON2bits.OCTRIG = 0x0;
	OC2CON1bits.OCTSEL = 0b011;	// timer 5 as source
	OC2CON1bits.OCM = 0b110;	//Edge-Aligned PWM mode on OCx

	// Timers clock input = Fosc/2 = 2 MHz
	// Period time = PR * 1/(Fosc/2) => PR = period time * Fosc/2
	// We need a period time of 250 us (4 KHz) => 0.00025 * 2000000 = 500
	// We need a period time of 20 us (50 KHz) => 0.00002 * 2000000 = 40
	// We need a period time of 10 ms (100 Hz) => 0.01 * 2000000 = 20000
	// We need a period time of 33 ms (30 Hz) => 0.033 * 2000000 = 66000 (overflow, we use 65000 and we get about 30.7 Hz)
	T2CON = 0;					// reset Timer2 CONtrol register
	T2CONbits.TCKPS = 0;		// precsaler = 1
	TMR2 = 0;					// clear timer 2
	PR2 = PERIOD_MOTORS;
	IFS0bits.T2IF = 0;			// clear interrupt flag
	IEC0bits.T2IE = 1;			// set interrupt enable bit
	T2CONbits.TON = 1;			// start Timer2

	T5CON = 0;					// reset Timer5 CONtrol register
	T5CONbits.TCKPS = 0;		// precsaler = 1
	TMR5 = 0;					// clear timer 5
	PR5 = PERIOD_MOTORS;
	IFS1bits.T5IF = 0;			// clear interrupt flag
	IEC1bits.T5IE = 1;			// set interrupt enable bit
	T5CONbits.TON = 1;			// start Timer5

	// RGB pwms
	value1_pwm = 64;			// initial duty cycle (leds turned off when duty=period)
	value2_pwm = 64;			// initial period, see timer 3 (PR3) configuration
	config1_pwm = 0;
   	config1_pwm = OC_IDLE_CON | OC_TIMER3_SRC | CMP_FAULT2_IN_DISABLE | OC_FAULT1_IN_DISABLE | OC_FAULT0_IN_DISABLE | OC_PWM_EDGE_ALIGN ;
	config2_pwm = OC_SYNC_ENABLE | 0x001F;
	OpenOC3_GB(config1_pwm, config2_pwm, value1_pwm, value2_pwm);
	OpenOC4_GB(config1_pwm, config2_pwm, value1_pwm, value2_pwm);
	OpenOC5_GB(config1_pwm, config2_pwm, value1_pwm, value2_pwm);

	// turn on timer 3 as clock source for leds pwm (pwm3, pwm4, pwm5)
	// Timer3 clock input = Fosc/2 = 2 MHz
	// Period freq = (Fosc/2)/PR => PR = (Fosc/2)/period freq
	// We need a frequency of about 30 KHz => 2000000/30000 = 67
	// since the values for the leds goes from 0 to 128, we choose PR=64 => period freq = 2000000/64 = 31250 Hz
	T3CON = 0;							// reset Timer3 CONtrol register
	T3CONbits.TCKPS = 0;				// precsaler = 1
	TMR3 = 0;							// clear timer 3
	PR3 = 64;
	IFS0bits.T3IF = 0;					// clear interrupt flag
	IEC0bits.T3IE = 0;					// set interrupt enable bit
	T3CONbits.TON = 1;					// start Timer1

	// INT0 inerrupt configuration (IR commands receiving)
	e_init_remote_control();
	e_start_agendas_processing();

	// adc configuration
	CloseADC10();
	AD1CHS = 0;
	config1 = ADC_MODULE_ON | ADC_FORMAT_INTG | ADC_CLK_AUTO |ADC_AUTO_SAMPLING_ON ;
	config2 = ADC_SCAN_ON | ADC_INTR_8_CONV; // 7 samples: 3 for acc, 4 for motors current consumption, 1 for battery
	config3 = ADC_SAMPLE_TIME_1 | ADC_CONV_CLK_8Tcy;	// min Tad = 75 ns
	configport = ENABLE_AN0_ANA | ENABLE_AN1_ANA | ENABLE_AN4_ANA | ENABLE_AN5_ANA | ENABLE_AN6_ANA | ENABLE_AN7_ANA | ENABLE_AN8_ANA | ENABLE_AN11_ANA;
	configscan = ADC_SCAN_AN0 | ADC_SCAN_AN1 | ADC_SCAN_AN4 | ADC_SCAN_AN5 | ADC_SCAN_AN6 | ADC_SCAN_AN7 | ADC_SCAN_AN8 | ADC_SCAN_AN11;
	//configport = ENABLE_AN6_ANA | ENABLE_AN7_ANA;
	//configscan = ADC_SCAN_AN6 | ADC_SCAN_AN7;
	//configport = ENABLE_AN7_ANA;
	//configscan = ADC_SCAN_AN7;
	//configport = ENABLE_AN5_ANA | ENABLE_AN6_ANA | ENABLE_AN7_ANA | ENABLE_AN8_ANA ;
	//configscan = ADC_SCAN_AN5 | ADC_SCAN_AN6 | ADC_SCAN_AN7 | ADC_SCAN_AN8;
	OpenADC10(config1,config2,config3,configport,configscan);
	IFS0bits.AD1IF = 0;      			//Clear the A/D interrupt flag bit
	IEC0bits.AD1IE = 1;      			//Set the A/D interrupt enable bit


	// timer1 configuration, used for timing measures and debugging
	T1CON = 0;							// reset Timer1 CONtrol register
	T1CONbits.TCKPS = 0;				// precsaler = 1
	TMR1 = 0;							// clear timer 2
	PR1 = 0xFFFF;						// interrupt every 32768 us
	IFS0bits.T1IF = 0;					// clear interrupt flag
	IEC0bits.T1IE = 1;					// set interrupt enable bit
	TMR1 = 0;							// clear timer 2
	PR1 = 30000;						// We need a period time of 5 ms (200 Hz) => 0.005 * 2000000 = 10000
	IFS0bits.T1IF = 0;					// clear interrupt flag
	IEC0bits.T1IE = 0;					// set interrupt enable bit
	T1CONbits.TON = 0;
}


void __attribute__((interrupt, auto_psv)) _T1Interrupt(void)
{
	IFS0bits.T1IF = 0;
	//time_counter++;

	debug_data_arr[0] = ADCResult[3];	// motleftB
	debug_data_arr[1] = ADCResult[4];	// motleftA
	debug_data_arr[2] = ADCResult[5];	// batt
	debug_data_arr[3] = 0xFFFF;
	fwrite(debug_data_arr, sizeof(signed int), 4, stdout);

}

// pwm left timer
void __attribute__((interrupt, auto_psv)) _T2Interrupt(void)
{
	IFS0bits.T2IF = 0;
	exec_motion_controller = 1;
	start_left_current_sampling = 1;
	start_left_vel_sampling = 0;
	last_left_vel_sum = left_vel_sum;
	last_num_lvel_samples_avg = num_lvel_samples_avg;
	left_vel_sum = 0;
	num_lvel_samples_avg = 0;
	last_tmr2_value = tmr2_value;
	compute_left_vel = 1;

	if(pwm_left >= 0) {   // move forward
		PPSUnLock; 
		iPPSOutput(OUT_PIN_PPS_RP11, OUT_FN_PPS_NULL);
		_LATB11 = 0;
		iPPSOutput(OUT_PIN_PPS_RP15, OUT_FN_PPS_OC1);
		PPSLock;
	} else if(pwm_left < 0) {         // move backward
		PPSUnLock;
		iPPSOutput(OUT_PIN_PPS_RP15, OUT_FN_PPS_NULL);
		_LATB15 = 0;
		iPPSOutput(OUT_PIN_PPS_RP11, OUT_FN_PPS_OC1);
		PPSLock;
	}

}

// pwm right timer
void __attribute__((interrupt, auto_psv)) _T5Interrupt(void)
{
	IFS1bits.T5IF = 0;
	start_right_current_sampling = 1;
	start_right_vel_sampling = 0;
	last_right_vel_sum = right_vel_sum;
	last_num_rvel_samples_avg = num_rvel_samples_avg;
	right_vel_sum = 0;
	num_rvel_samples_avg = 0;
	last_tmr5_value = tmr5_value;
	compute_right_vel = 1;

	if(pwm_right >= 0) {   // move forward
		PPSUnLock;
		iPPSOutput(OUT_PIN_PPS_RP10, OUT_FN_PPS_NULL);
		_LATB10 = 0;
		iPPSOutput(OUT_PIN_PPS_RP14, OUT_FN_PPS_OC2);
		PPSLock;
	} else if(pwm_right < 0) {         // move backward
		PPSUnLock;
		iPPSOutput(OUT_PIN_PPS_RP14, OUT_FN_PPS_NULL);		
		_LATB14 = 0;
		iPPSOutput(OUT_PIN_PPS_RP10, OUT_FN_PPS_OC2);
		PPSLock;
	}

}

void __attribute__((__interrupt__, auto_psv)) _ADC1Interrupt(void)
{
	volatile unsigned int * adc_ptr;
   	IFS0bits.AD1IF = 0;
	adc_ptr = &ADC1BUF0;

	ADCResult[0] = *adc_ptr++;	// acc x
	ADCResult[1] = *adc_ptr++;	// acc y
	ADCResult[2] = *adc_ptr++;	// acc z
	ADCResult[3] = *adc_ptr++;	// motright A
	ADCResult[4] = *adc_ptr++;	// motleft B
	ADCResult[5] = *adc_ptr++;	// motleft A
	ADCResult[6] = *adc_ptr++;	// motright B
	ADCResult[7] = *adc_ptr++;	// battery

	battery_voltage = ADCResult[7];

	if(start_calibration) {
		if(num_samples<50) {
			acc_samples[0][num_samples] = ADCResult[0];
			acc_samples[1][num_samples] = ADCResult[1];
			acc_samples[2][num_samples] = ADCResult[2];
			num_samples++;
		} else if(num_samples==50) {
			samples_received=1;
		}
	}

	if(calibration_done) {
		acc_x_sum += ADCResult[0];
		acc_y_sum += ADCResult[1];
		acc_z_sum += ADCResult[2];
		num_samples_avg++;
		if(num_samples_avg==NUM_SAMPLES) {
			last_acc_x = acc_x_sum>>6;	// divide by 64
			last_acc_y = acc_y_sum>>6;
			last_acc_z = acc_z_sum>>6;
			acc_x_sum = 0;
			acc_y_sum = 0;
			acc_z_sum = 0;
			num_samples_avg=0;	
		}
	}


	if(start_left_current_sampling) {
		if(TMR2 <= OC1R) {		// perform sampling from period init to duty cycle
			if(pwm_left>=0) {
				left_current_avg += ADCResult[4];
			} else {
				left_current_avg += ADCResult[5];
			}
			left_current_avg = left_current_avg>>1;
		} else {
			last_left_current = left_current_avg;
			left_current_avg = 0;
			start_left_current_sampling = 0;
			num_left_samples_discarded = 0;
			start_left_vel_sampling = 1;
		}
	}

	if(start_left_vel_sampling) { // && !compute_left_vel) {
		//if(TMR2 > OC1R && num_lvel_samples_avg<MAX_VEL_SAMPLES) {
		//if(num_lvel_samples_avg<MAX_VEL_SAMPLES && (num_left_samples_discarded++ > NUM_SAMPLES_TO_DISCARD)) {
		//	tmr2_value = TMR2;
			//compute_left_vel = 1;
			if(pwm_left>=0) {
				left_vel_sum += ADCResult[5];
			} else {
				left_vel_sum += ADCResult[4];
			}
			num_lvel_samples_avg++;
		//}
	} 

	if(start_right_current_sampling) {
		if(TMR5 <= OC2R) {
			if(pwm_right>=0) {
				right_current_avg += ADCResult[6];
			} else {
				right_current_avg += ADCResult[3];
			}
			right_current_avg = right_current_avg>>1;
		} else {
			last_right_current = right_current_avg;
			right_current_avg = 0;
			start_right_current_sampling = 0;
			num_right_samples_discarded = 0;
			start_right_vel_sampling = 1;
		}
	}

	if(start_right_vel_sampling) { // && !compute_right_vel) {
		//if(TMR5 > OC2R && num_rvel_samples_avg<=MAX_VEL_SAMPLES) {
		//if(num_rvel_samples_avg<=MAX_VEL_SAMPLES && (num_right_samples_discarded++ > NUM_SAMPLES_TO_DISCARD)) {
		//	tmr5_value = TMR5;
			//compute_right_vel = 1;
			if(pwm_right>=0) {
				right_vel_sum += ADCResult[3];
			} else {
				right_vel_sum += ADCResult[6];
			}
			num_rvel_samples_avg++;
		//}
	} 

}

void __attribute__ ((interrupt,auto_psv)) _U1RXInterrupt(void)
{
	U1RX_Clear_Intr_Status_Bit;
	while(!DataRdyUART1());
	Rxdata[count++] = ReadUART1();
	if(count == 2)
		DataAvailable=1;
}

void __attribute__ ((interrupt,auto_psv)) _U1TXInterrupt(void)
{
	U1TX_Clear_Intr_Status_Bit;
}

void calibrate_accelerometer() {
	int j=0;
	acc_x_offset = 0;
	acc_y_offset = 0;
	acc_z_offset = 0;

	for(j=0; j<50; j++) {
		acc_x_offset += acc_samples[0][j];
		acc_y_offset += acc_samples[1][j];
		acc_z_offset += acc_samples[2][j];
	}
	acc_x_offset = acc_x_offset/50;
	acc_y_offset = acc_y_offset/50;
	acc_z_offset = acc_z_offset/50;
	gravity = acc_z_offset - (acc_x_offset+acc_y_offset)/2; // + 20;	//512/3;
	acc_z_offset = acc_z_offset - gravity;

	//printf("acc_x_offset = %d\r\n", acc_x_offset);
	//printf("acc_y_offset = %d\r\n", acc_y_offset);
	//printf("acc_z_offset = %d\r\n", acc_z_offset);
	//printf("gravity = %d\r\n", gravity);

	calibration_done = 1;

}

int getselector() {
   return _RA2 + 2*_RA3+ 4*_RA8 + 8*_RB4;
}

void motion_controller() {

	acc_x = last_acc_x-acc_x_offset;
	acc_y = last_acc_y-acc_y_offset;
	acc_z = last_acc_z-acc_z_offset;

	if(acc_x < 0) {
		abs_acc_x = -acc_x;
	} else {
		abs_acc_x = acc_x;
	}
	if(acc_y < 0) {
		abs_acc_y = -acc_y;
	} else {
		abs_acc_y = acc_y;
	}
	if(acc_z < 0) {
		abs_acc_z = -acc_z;
	} else {
		abs_acc_z = acc_z;
	}

	if(gravity < abs_acc_x) {
		gravity = abs_acc_x;
	}
	if(gravity < abs_acc_y) {
		gravity = abs_acc_y;
	}
	if(gravity < abs_acc_z) {
		gravity = abs_acc_z;
	}	

	current_angle = atan2(((float)acc_x/(float)gravity),((float)acc_y/(float)gravity))*180.0/PI;
	if(current_angle<0) {
		current_angle = 360+current_angle;	// angles from 0 to 360
	}

	//printf("abs_acc_z = %d\n", abs_acc_z);
	//if(abs_acc_z>10 || !start_motion_controller)	// not vertical
	if(!start_motion_controller) {
		if(getselector()!=2) {
			exec_current_controller = 1;
		}
		pwm_right_motion_controller = pwm_right_desired;
		pwm_left_motion_controller = pwm_left_desired;
		return;
	}

	delta_angles[1] = delta_angles[0];
	delta_angle = desired_orientation-current_angle;
	// set delta angle to be in the range -180..180
	if(delta_angle < -180) {
		delta_angle += 360;
	}
	if(delta_angle > 180) {
		delta_angle -= 360;
	}
	delta_angles[0] = delta_angle;
//	if(delta_angle < 1 && delta_angle > -1) {
//		delta_angles_sum = 0;
//	}
	delta_angles_sum += delta_angle;
	if(delta_angles_sum > 45) {		// maximum error is 180 degrees
		delta_angles_sum = 45;
	} else if(delta_angles_sum < -45) {
		delta_angles_sum = -45;
	}

	//printf("delta angle = %d\r\n", delta_angle);

	// PD controller: new_pwm = pwm_desired + P*(error) + D*(error-previous_error)
	pwm_right_motion_controller = pwm_right_desired + (signed long int)((float)P*last_right_vel/512.0*((float)delta_angle*DELTA_ANGLE_FACT) + (float)D*last_right_vel/512.0*((float)(delta_angles[0]-delta_angles[1])*DELTA_ANGLE_FACT) + (float)I*((float)delta_angles_sum*DELTA_ANGLE_FACT));
	pwm_left_motion_controller = pwm_left_desired - (signed long int)((float)P*last_left_vel/512.0*((float)delta_angle*DELTA_ANGLE_FACT) - (float)D*last_left_vel/512.0*((float)(delta_angles[0]-delta_angles[1])*DELTA_ANGLE_FACT) -  (float)I*((float)delta_angles_sum*DELTA_ANGLE_FACT));		

	if(pwm_right_motion_controller < 0 && pwm_right_desired > 0) {
		pwm_right_motion_controller = 0;
	} else if(pwm_right_motion_controller > 0 && pwm_right_desired < 0 ) {
		pwm_right_motion_controller = 0;
	}

	if(pwm_left_motion_controller < 0 && pwm_left_desired > 0) {
		pwm_left_motion_controller = 0;
	} else if(pwm_left_motion_controller > 0 && pwm_left_desired < 0 ) {
		pwm_left_motion_controller = 0;
	}

	if (pwm_right_motion_controller>MAX_PWM) pwm_right_motion_controller=MAX_PWM;
	if (pwm_left_motion_controller>MAX_PWM) pwm_left_motion_controller=MAX_PWM;
	if (pwm_right_motion_controller<-MAX_PWM) pwm_right_motion_controller=-MAX_PWM;
	if (pwm_left_motion_controller<-MAX_PWM) pwm_left_motion_controller=-MAX_PWM;

	exec_current_controller = 1;

}

void current_controller() {
	if(getselector()==2 || getselector()==3) {
		if(getselector()==2) {
			pwm_right_working = pwm_right_speed_controller;
			pwm_left_working = pwm_left_speed_controller;
		} else {
			pwm_right_working = pwm_right_motion_controller;
			pwm_left_working = pwm_left_motion_controller;
		}
		if(pwm_left_working < 0) {
			pwm_current_lim = ((CURRENT_LIM_UP+(100*pwm_left_current_controller/PERIOD_MOTORS))>>1);	// gives the current consumption limit for the pwm value specified at last update
			if(last_left_current > pwm_current_lim && (pwm_left_working < -PWM_THRESHOLD)) {	// if the consumption is too high and pwm value is also high
				pwm_current_step = ((MAX_CURRENT_MEAS-pwm_current_lim)*PERIOD_MOTORS/100)/4*3;	//>>1
				pwm_left_current_controller = pwm_left_working + pwm_current_step;
				//pwm_left_current_controller = (signed long int)((double)pwm_left_working/(double)((double)-pwm_left_current_controller/(double)pwm_current_lim));
			} else {																					// the consumption could be high if the pwm is low in order for the robot to be able to start moving at least
				pwm_left_current_controller = pwm_left_working;
			}
		} else if (pwm_left_working > 0) {
			pwm_current_lim = ((CURRENT_LIM_UP-(100*pwm_left_current_controller/PERIOD_MOTORS))>>1);
			if(last_left_current > pwm_current_lim && (pwm_left_working > PWM_THRESHOLD)) {
				pwm_current_step = ((MAX_CURRENT_MEAS-pwm_current_lim)*PERIOD_MOTORS/100)/4*3;
				pwm_left_current_controller = pwm_left_working - pwm_current_step;	//pwm_left_current_controller>>1;
				//pwm_left_current_controller = (signed long int)((double)pwm_left_working/(double)((double)pwm_left_current_controller/(double)pwm_current_lim));
			} else {
				pwm_left_current_controller = pwm_left_working;
			}			
		} else {
			pwm_left_current_controller = pwm_left_working;
		}
	
		if(pwm_right_working < 0) {
			pwm_current_lim = ((CURRENT_LIM_UP+(100*pwm_right_current_controller/PERIOD_MOTORS))>>1);
			if(last_right_current > pwm_current_lim && (pwm_right_working < -PWM_THRESHOLD)) {
				pwm_current_step = ((MAX_CURRENT_MEAS-pwm_current_lim)*PERIOD_MOTORS/100)/4*3;
				pwm_right_current_controller = pwm_right_working + pwm_current_step;
				//pwm_right_current_controller = (signed long int)((double)pwm_right_working/(double)((double)-pwm_right_current_controller/(double)pwm_current_lim));
			} else {
				pwm_right_current_controller = pwm_right_working;
			}
		} else if (pwm_right_working > 0) {
			pwm_current_lim = ((CURRENT_LIM_UP-(100*pwm_right_current_controller/PERIOD_MOTORS))>>1);
			if(last_right_current > pwm_current_lim && (pwm_right_working > PWM_THRESHOLD)) {
				pwm_current_step = ((MAX_CURRENT_MEAS-pwm_current_lim)*PERIOD_MOTORS/100)/4*3;
				pwm_right_current_controller = pwm_right_working - pwm_current_step;	//pwm_right_current_controller>>1;
				//pwm_right_current_controller = (signed long int)((double)pwm_right_working/(double)((double)pwm_right_current_controller/(double)pwm_current_lim));
			} else {
				pwm_right_current_controller = pwm_right_working;
			}			
		} else {
			pwm_right_current_controller = pwm_right_working;
		}
	
		if (pwm_right_current_controller>MAX_PWM) pwm_right_current_controller=MAX_PWM;
		if (pwm_left_current_controller>MAX_PWM) pwm_left_current_controller=MAX_PWM;
		if (pwm_right_current_controller<-MAX_PWM) pwm_right_current_controller=-MAX_PWM;
		if (pwm_left_current_controller<-MAX_PWM) pwm_left_current_controller=-MAX_PWM;
	} else {
		pwm_left_current_controller = pwm_left_motion_controller;
		pwm_right_current_controller = pwm_right_motion_controller;
	}
	update_pwm = 1;

}

int duty_to_percentage(signed long int pwm) {
	if(pwm>=0) {
		return (int)((float)pwm/(float)PERIOD_MOTORS*100.0);
	} else {
		return (int)((float)-pwm/(float)PERIOD_MOTORS*100.0);
	}
}

void speed_controller() {

	if(pwm_right_desired==0 || getselector()!=2) {
		pwm_right_speed_controller = pwm_right_desired;
	} else {
		delta_right_speeds[1] = delta_right_speeds[0];
		//right_adc_to_duty = last_right_vel*39;	// linear trendline
		right_adc_to_duty = (signed long int)(((float)(last_right_vel*last_right_vel))*0.058 - (float)(last_right_vel)*1.145);
		if(pwm_right_desired>=0) {
			delta_right_speed = pwm_right_desired - right_adc_to_duty; //-9925); //(last_right_vel>>3)*650;
		} else {
			delta_right_speed = pwm_right_desired + right_adc_to_duty; //-9925); //(last_right_vel>>3)*650;
		}
		delta_right_speeds[0] = delta_right_speed;
		delta_right_speed_sum += delta_right_speed;
		if(delta_right_speed_sum > (MAX_PWM/2)) {
			delta_right_speed_sum = (MAX_PWM/2);
			// turn on cyan
			SetDCOC3PWM_GB(0,64);
			SetDCOC4PWM_GB(0,64);
			SetDCOC5PWM_GB(64,64);
		} else if(delta_right_speed_sum < -(MAX_PWM/2)) {
			delta_right_speed_sum = -(MAX_PWM/2);
			// turn on pink
			SetDCOC3PWM_GB(0,64);
			SetDCOC4PWM_GB(64,64);
			SetDCOC5PWM_GB(0,64);
		}

		right_increment = (signed long int)((float)P_SPEED*((float)delta_right_speed) + (float)D_SPEED*((float)(delta_right_speeds[0]-delta_right_speeds[1])) + (float)I_SPEED*(float)delta_right_speed_sum);
		//p_speed_prop = pwm_right_desired/PERIOD_MOTORS;
		//i_speed_prop = pwm_right_desired/(PERIOD_MOTORS*2.0);
		//right_increment = (signed long int)((float)p_speed_prop*((float)delta_right_speed) + (float)D_SPEED*((float)(delta_right_speeds[0]-delta_right_speeds[1])) + (float)i_speed_prop*(float)delta_right_speed_sum);

		//if(pwm_right_desired>=0) {
			pwm_right_speed_controller = pwm_right_desired + right_increment;
		//} else {
		//	pwm_right_speed_controller = pwm_right_desired - (signed int)((float)P_SPEED*((float)delta_right_speed) - (float)D_SPEED*((float)(delta_right_speeds[0]-delta_right_speeds[1])));
		//}

		if(pwm_right_speed_controller < 0 && pwm_right_desired >= 0) {
			pwm_right_speed_controller = 0;
		}
		if(pwm_right_speed_controller > 0 && pwm_right_desired < 0 ) {
			pwm_right_speed_controller = 0;
		}

		if (pwm_right_speed_controller>MAX_PWM) pwm_right_speed_controller=MAX_PWM;
		if (pwm_right_speed_controller<-MAX_PWM) pwm_right_speed_controller=-MAX_PWM;

	}


	if(pwm_left_desired==0 || getselector()!=2) {
		pwm_left_speed_controller = pwm_left_desired;
	} else {
		//left_adc_to_duty = (last_left_vel*37);
		left_adc_to_duty = (signed long int)(((float)(last_left_vel*last_left_vel))*0.062 - (float)(last_left_vel)*3.188);
		delta_left_speeds[1] = delta_left_speeds[0];
		if(pwm_left_desired>=0) {
			delta_left_speed = pwm_left_desired -  left_adc_to_duty; //-9925); //(last_left_vel>>3)*650;
		} else {
			delta_left_speed = pwm_left_desired +  left_adc_to_duty; //-9925); //(last_left_vel>>3)*650;
		}
		delta_left_speeds[0] = delta_left_speed;
		delta_left_speed_sum += delta_left_speed;
		if(delta_left_speed_sum > (MAX_PWM/2)) {
			delta_left_speed_sum = (MAX_PWM/2);
		} else if(delta_left_speed_sum < -(MAX_PWM/2)) {
			delta_left_speed_sum = -(MAX_PWM/2);
		}
	
		left_increment = (signed long int)((float)P_SPEED*((float)delta_left_speed) + (float)D_SPEED*((float)(delta_left_speeds[0]-delta_left_speeds[1])) + (float)I_SPEED*(float)delta_left_speed_sum);
		//p_speed_prop = pwm_left_desired/PERIOD_MOTORS;
		//i_speed_prop = pwm_left_desired/(PERIOD_MOTORS*2.0);
		//left_increment = (signed long int)((float)p_speed_prop*((float)delta_left_speed) + (float)D_SPEED*((float)(delta_left_speeds[0]-delta_left_speeds[1])) + (float)i_speed_prop*(float)delta_left_speed_sum);

//		if(pwm_left_desired >= 0) {
//			if(-left_increment > pwm_left_desired) {
//				left_increment = -pwm_left_desired;
//			}
//		} else {
//			if(left_increment < 
//		}

		//if(pwm_left_desired>=0) {
			pwm_left_speed_controller = pwm_left_desired + left_increment;
		//} else {
		//	pwm_left_speed_controller = pwm_left_desired - (signed int)((float)P_SPEED*((float)delta_left_speed) - (float)D_SPEED*((float)(delta_left_speeds[0]-delta_left_speeds[1])));
		//}

		if(pwm_left_speed_controller < 0 && pwm_left_desired >= 0) {
			pwm_left_speed_controller = 0;
		}
		if(pwm_left_speed_controller > 0 && pwm_left_desired < 0 ) {
			pwm_left_speed_controller = 0;
		}

		if (pwm_left_speed_controller>MAX_PWM) pwm_left_speed_controller=MAX_PWM;
		if (pwm_left_speed_controller<-MAX_PWM) pwm_left_speed_controller=-MAX_PWM;
	}	

	//update_pwm = 1;
	if(getselector()==2) {
		//exec_current_controller = 1;
		pwm_left_current_controller = pwm_left_speed_controller;
		pwm_right_current_controller = pwm_right_speed_controller;
		update_pwm = 1;
	}
}

void init_sequence() {

	if(init_pause_state<3) {
		switch(init_pause_state) {
			case 0: // turn on red
					SetDCOC3PWM_GB(64,64);
					SetDCOC4PWM_GB(64,64);
					SetDCOC5PWM_GB(0,64);
					break;
			case 1: // turn on green
					SetDCOC3PWM_GB(64,64);
					SetDCOC4PWM_GB(0,64);
					SetDCOC5PWM_GB(64,64);
					break;
			case 2: // turn on blue
					SetDCOC3PWM_GB(0,64);
					SetDCOC4PWM_GB(64,64);
					SetDCOC5PWM_GB(64,64);
					break;
		}
		init_pause_state++;
		return;
	} else if(init_pause_state==3) {
		start_calibration=1;
		init_pause_state++;
		return;
	}

}

void turn_off_battery_led() {
	SetDCOC3PWM_GB(64,64);
	SetDCOC4PWM_GB(64,64);
	SetDCOC5PWM_GB(64,64);
	e_destroy_agenda(turn_off_battery_led);
}

void robot_is_on() {
	if(pwm_red==64 && pwm_green==64 && pwm_blue==64) {
		SetDCOC3PWM_GB(64,64);
		SetDCOC4PWM_GB(64,64);
		SetDCOC5PWM_GB(0,64);
		e_activate_agenda(turn_off_battery_led, 2);
	}

	if(DEBUG_ASCII) {
		printf("rvel=%d\r\n", last_right_vel);
		printf("lvel=%d\r\n", last_left_vel);
		printf("rcurrent=%d\r\n", last_right_current);
		printf("lcurrent=%d\r\n", last_left_current);
		printf("pwmr=%ld\r\n", pwm_right_motion_controller);
		printf("pwml=%ld\r\n", pwm_left_motion_controller);
		//printf("x=%d\r\n", ADCResult[0]);
		//printf("y=%d\r\n", ADCResult[1]);
		//printf("z=%d\r\n", ADCResult[2]);
		printf("angle=%d\r\n", current_angle);
		printf("delta=%d\r\n", delta_angle);
		printf("sum=%d\r\n", delta_angles_sum);
		printf("batt=%d\r\n", battery_voltage);
		printf("\r\n");
	}
	if(DEBUG_MATLAB) {
		fwrite(&last_right_vel, sizeof(unsigned int), 1, stdout);
		fwrite(&last_left_vel, sizeof(unsigned int), 1, stdout);
		fwrite(&current_angle, sizeof(signed int), 1, stdout);
		fwrite(&battery_voltage, sizeof(unsigned int), 1, stdout);
	}
		
}

signed int right_pwm_to_speed(signed long int pwm) {
	return (signed int)(pwm/784);
}

signed int left_pwm_to_speed(signed long int pwm) {
	return (signed int)(pwm/804);
}

signed long int right_speed_to_pwm(signed int speed) {
	return (signed long int)(784*speed);
}

signed long int left_speed_to_pwm(signed int speed) {
	return (signed long int)(804*speed);
}

int main() {
	color_state=0;
	pwm_right = 0;
	pwm_left = 0;
	pwm_red = 64;
	pwm_green = 64;
	pwm_blue = 64;
	count=0;
	DataAvailable=0;

	init();					// peripherals configuration

	e_activate_agenda(robot_is_on, 20000);		// every 2 seconds
	e_activate_agenda(init_sequence, 10000);	// every second

	while(!samples_received);
	calibrate_accelerometer();
	SetDCOC3PWM_GB(64,64);
	SetDCOC4PWM_GB(64,64);
	SetDCOC5PWM_GB(64,64);

	_LATC6 = 1;
	init_rf_module(40);		// RF chip configuration


//	// flush the transmit buffer
//	toSend[0]=NRF_FLUSH_TX;
//	while(DataRdySPI1()) {
//		spi_data = ReadSPI1();
//	}
//	_LATC6 = 0;;	
//	WriteSPI1(toSend[0]);
//	while(!DataRdySPI1());
//	spi_data = ReadSPI1();
//	_LATC6 = 1;



	// write initial ack payload
	toSend[0]= NRF_W_ACK_PAYLOAD_P0;
	toSend[1]=0x00;
			while(DataRdySPI1()) {
				spi_data = ReadSPI1();
			}
	_LATC6 = 0;
	WriteSPI1(toSend[0]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	WriteSPI1(toSend[1]);
	while(!DataRdySPI1());
	spi_data = ReadSPI1();
	_LATC6 = 1;
	while(DataRdySPI1()) {
		spi_data = ReadSPI1();
	}


//			toSend[0]=NRF_W_ACK_PAYLOAD_P0;
//			toSend[1]=0x05;
//      		while(DataRdySPI1()) {
//        		spi_data = ReadSPI1();
//      		}	
//			_LATC6 = 0;
//         	WriteSPI1(toSend[0]);
//         	while(!DataRdySPI1());
//         	spi_data = ReadSPI1();
//			for(i=0; i<PAYLOAD_SIZE; i++) {
//            	WriteSPI1(toSend[1]);
//            	while(!DataRdySPI1());
//            	spi_data = ReadSPI1();
//			}
//			_LATC6 = 1;


    while(1) {

		if(DataAvailable) {
			DataAvailable=0;
			count=0;
			switch(Rxdata[0]) {

				case 's': 	// stop motors
					pwm_right_desired = 0;
					pwm_left_desired = 0;
					desired_orientation = current_angle;
					//if(ACTIVATE_MOTION_CONTROLLER)
						start_motion_controller = 0;
                  	break;
				case 'w':	// both motors forward
					if(pwm_right_desired > pwm_left_desired) {
						pwm_left_desired = pwm_right_desired;
					} else {
						pwm_right_desired = pwm_left_desired;
					}
					pwm_right_desired += STEP_MOTORS;
					pwm_left_desired += STEP_MOTORS;
                	if (pwm_right_desired>MAX_PWM) pwm_right_desired=MAX_PWM;
                	if (pwm_left_desired>MAX_PWM) pwm_left_desired=MAX_PWM;
					desired_orientation = current_angle;
					//if(ACTIVATE_MOTION_CONTROLLER)
					//	start_motion_controller = 1;
					if(getselector()==1 || getselector()==3) {
						start_motion_controller = 1;
					} else {
						start_motion_controller = 0;
					}
                  	break;
				case 'x':	// both motors backward
					if(pwm_right_desired < pwm_left_desired) {
						pwm_left_desired = pwm_right_desired;
					} else {
						pwm_right_desired = pwm_left_desired;
					}
					pwm_right_desired -= STEP_MOTORS;
					pwm_left_desired -= STEP_MOTORS;
                	if (pwm_right_desired<-MAX_PWM) pwm_right_desired=-MAX_PWM;
                	if (pwm_left_desired<-MAX_PWM) pwm_left_desired=-MAX_PWM;
					desired_orientation = current_angle;
					//if(ACTIVATE_MOTION_CONTROLLER)
					//	start_motion_controller = 1;
					if(getselector()==1 || getselector()==3) {
						start_motion_controller = 1;
					} else {
						start_motion_controller = 0;
					}
                  	break;
				case 'd':	// both motors right
					pwm_right_desired -= STEP_MOTORS;
					pwm_left_desired += STEP_MOTORS;
                	if (pwm_right_desired<-MAX_PWM) pwm_right_desired=-MAX_PWM;
                	if (pwm_left_desired>MAX_PWM) pwm_left_desired=MAX_PWM;
					desired_orientation = current_angle;
					//if(ACTIVATE_MOTION_CONTROLLER)
					//	start_motion_controller = 1;
					if(getselector()==1 || getselector()==3) {
						start_motion_controller = 1;
					} else {
						start_motion_controller = 0;
					}
					break;
				case 'a':	// both motors left
					pwm_right_desired += STEP_MOTORS;
					pwm_left_desired -= STEP_MOTORS;
                	if (pwm_right_desired>MAX_PWM) pwm_right_desired=MAX_PWM;
                	if (pwm_left_desired<-MAX_PWM) pwm_left_desired=-MAX_PWM;
					desired_orientation = current_angle;
					//if(ACTIVATE_MOTION_CONTROLLER)
					//	start_motion_controller = 1;
					if(getselector()==1 || getselector()==3) {
						start_motion_controller = 1;
					} else {
						start_motion_controller = 0;
					}
					break;
				case 'q':	// left motor forward
					pwm_left_desired += STEP_MOTORS;
                	if (pwm_left_desired>MAX_PWM) pwm_left_desired=MAX_PWM;
					desired_orientation = current_angle;
					//if(ACTIVATE_MOTION_CONTROLLER)
					//	start_motion_controller = 1;
					if(getselector()==1 || getselector()==3) {
						start_motion_controller = 1;
					} else {
						start_motion_controller = 0;
					}
					break;
				case 'e':	// right motor forward
					pwm_right_desired += STEP_MOTORS;
                	if (pwm_right_desired>MAX_PWM) pwm_right_desired=MAX_PWM;
					desired_orientation = current_angle;
					//if(ACTIVATE_MOTION_CONTROLLER)
					//	start_motion_controller = 1;
					if(getselector()==1 || getselector()==3) {
						start_motion_controller = 1;
					} else {
						start_motion_controller = 0;
					}
					break;
				case 'y':	// left motor backward
					pwm_left_desired -= STEP_MOTORS;
                	if (pwm_left_desired<-MAX_PWM) pwm_left_desired=-MAX_PWM;
					desired_orientation = current_angle;
					//if(ACTIVATE_MOTION_CONTROLLER)
					//	start_motion_controller = 1;
					if(getselector()==1 || getselector()==3) {
						start_motion_controller = 1;
					} else {
						start_motion_controller = 0;
					}
					break;
				case 'c':	// right motor backward
					pwm_right_desired -= STEP_MOTORS;
                	if (pwm_right_desired<-MAX_PWM) pwm_right_desired=-MAX_PWM;
					desired_orientation = current_angle;
					//if(ACTIVATE_MOTION_CONTROLLER)
					//	start_motion_controller = 1;
					if(getselector()==1 || getselector()==3) {
						start_motion_controller = 1;
					} else {
						start_motion_controller = 0;
					}
					break;
				case 'j':	// increases brightness
					pwm_red -= 8;
					if(pwm_red<0) pwm_red=0;
					pwm_green -= 8;
					if(pwm_green<0) pwm_green=0;
					pwm_blue -= 8;
					if(pwm_blue<0) pwm_blue=0;
					break;
				case 'l':	// decreases brightness
					pwm_red += 8;
					if(pwm_red>56) pwm_red=56;
					pwm_green += 8;
					if(pwm_green>56) pwm_green=56;
					pwm_blue += 8;
					if(pwm_blue>56) pwm_blue=56;
					break;
				case 'k':
					color_state++;
					if(color_state==5) {
						color_state=0;
					}	
					break;					
				default:
					break;
			}

			if(color_state==0) {		// turn on blue
				SetDCOC3PWM_GB(pwm_blue,64);
				SetDCOC4PWM_GB(64,64);
				SetDCOC5PWM_GB(64,64);						
			} else if(color_state==1) {	// turn on green
				SetDCOC3PWM_GB(64,64);	
				SetDCOC4PWM_GB(pwm_green,64);
				SetDCOC5PWM_GB(64,64);
			} else if(color_state==2) {	// turn on red
				SetDCOC3PWM_GB(64,64);
				SetDCOC4PWM_GB(64,64);
				SetDCOC5PWM_GB(pwm_red,64);
			} else if(color_state==3) {	// turn on white
				SetDCOC3PWM_GB(pwm_blue,64);
				SetDCOC4PWM_GB(pwm_green,64);
				SetDCOC5PWM_GB(pwm_red,64);
			} else if(color_state==4) {	// turn off
				SetDCOC3PWM_GB(64,64);
				SetDCOC4PWM_GB(64,64);
				SetDCOC5PWM_GB(64,64);
			}

		}

		ir_move = e_get_data();

		if(command_received) {
			command_received = 0;
			//printf("ir_move = \t%.2x\n", ir_move);

			switch(ir_move) {
				case 5: 	// stop motors
					pwm_right_desired = 0;
					pwm_left_desired = 0;
					speed_right_desired = 0;
					speed_left_desired = 0,
					desired_orientation = current_angle;
					//if(ACTIVATE_MOTION_CONTROLLER)
						start_motion_controller = 0;
                  	break;
				case 2:	// both motors forward
					if(pwm_right_desired > pwm_left_desired) {
						pwm_left_desired = pwm_right_desired;
					} else {
						pwm_right_desired = pwm_left_desired;
					}
					pwm_right_desired += STEP_MOTORS;
					pwm_left_desired += STEP_MOTORS;
                	if (pwm_right_desired>MAX_PWM) pwm_right_desired=MAX_PWM;
                	if (pwm_left_desired>MAX_PWM) pwm_left_desired=MAX_PWM;
					speed_right_desired = right_pwm_to_speed(pwm_right_desired);
					speed_left_desired = left_pwm_to_speed(pwm_left_desired);
					desired_orientation = current_angle;
					//if(ACTIVATE_MOTION_CONTROLLER)
					//	start_motion_controller = 1;
					if(getselector()==1 || getselector()==3) {
						start_motion_controller = 1;
					} else {
						start_motion_controller = 0;
					}
                  	break;
				case 8:	// both motors backward
					if(pwm_right_desired < pwm_left_desired) {
						pwm_left_desired = pwm_right_desired;
					} else {
						pwm_right_desired = pwm_left_desired;
					}
					pwm_right_desired -= STEP_MOTORS;
					pwm_left_desired -= STEP_MOTORS;
                	if (pwm_right_desired<-MAX_PWM) pwm_right_desired=-MAX_PWM;
                	if (pwm_left_desired<-MAX_PWM) pwm_left_desired=-MAX_PWM;
					speed_right_desired = right_pwm_to_speed(pwm_right_desired);
					speed_left_desired = left_pwm_to_speed(pwm_left_desired);
					desired_orientation = current_angle;
					//if(ACTIVATE_MOTION_CONTROLLER)
					//	start_motion_controller = 1;
					if(getselector()==1 || getselector()==3) {
						start_motion_controller = 1;
					} else {
						start_motion_controller = 0;
					}
                  	break;
				case 6:	// both motors right
					pwm_right_desired -= STEP_MOTORS;
					pwm_left_desired += STEP_MOTORS;
                	if (pwm_right_desired<-MAX_PWM) pwm_right_desired=-MAX_PWM;
                	if (pwm_left_desired>MAX_PWM) pwm_left_desired=MAX_PWM;
					speed_right_desired = right_pwm_to_speed(pwm_right_desired);
					speed_left_desired = left_pwm_to_speed(pwm_left_desired);
					desired_orientation = current_angle;
					//if(ACTIVATE_MOTION_CONTROLLER)
					//	start_motion_controller = 1;
					if(getselector()==1 || getselector()==3) {
						start_motion_controller = 1;
					} else {
						start_motion_controller = 0;
					}
					break;
				case 4:	// both motors left
					pwm_right_desired += STEP_MOTORS;
					pwm_left_desired -= STEP_MOTORS;
                	if (pwm_right_desired>MAX_PWM) pwm_right_desired=MAX_PWM;
                	if (pwm_left_desired<-MAX_PWM) pwm_left_desired=-MAX_PWM;
					speed_right_desired = right_pwm_to_speed(pwm_right_desired);
					speed_left_desired = left_pwm_to_speed(pwm_left_desired);
					desired_orientation = current_angle;
					//if(ACTIVATE_MOTION_CONTROLLER)
					//	start_motion_controller = 1;
					if(getselector()==1 || getselector()==3) {
						start_motion_controller = 1;
					} else {
						start_motion_controller = 0;
					}
					break;
				case 3:	// left motor forward
					pwm_left_desired += STEP_MOTORS;
                	if (pwm_left_desired>MAX_PWM) pwm_left_desired=MAX_PWM;
					speed_left_desired = left_pwm_to_speed(pwm_left_desired);
					desired_orientation = current_angle;
					//if(ACTIVATE_MOTION_CONTROLLER)
					//	start_motion_controller = 1;
					if(getselector()==1 || getselector()==3) {
						start_motion_controller = 1;
					} else {
						start_motion_controller = 0;
					}
					break;
				case 1:	// right motor forward
					pwm_right_desired += STEP_MOTORS;
                	if (pwm_right_desired>MAX_PWM) pwm_right_desired=MAX_PWM;
					speed_right_desired = right_pwm_to_speed(pwm_right_desired);
					desired_orientation = current_angle;
					//if(ACTIVATE_MOTION_CONTROLLER)
					//	start_motion_controller = 1;
					if(getselector()==1 || getselector()==3) {
						start_motion_controller = 1;
					} else {
						start_motion_controller = 0;
					}
					break;
				case 9:	// left motor backward
					pwm_left_desired -= STEP_MOTORS;
                	if (pwm_left_desired<-MAX_PWM) pwm_left_desired=-MAX_PWM;
					speed_left_desired = left_pwm_to_speed(pwm_left_desired);
					desired_orientation = current_angle;
					//if(ACTIVATE_MOTION_CONTROLLER)
					//	start_motion_controller = 1;
					if(getselector()==1 || getselector()==3) {
						start_motion_controller = 1;
					} else {
						start_motion_controller = 0;
					}
					break;
				case 7:	// right motor backward
					pwm_right_desired -= STEP_MOTORS;
                	if (pwm_right_desired<-MAX_PWM) pwm_right_desired=-MAX_PWM;
					speed_right_desired = right_pwm_to_speed(pwm_right_desired);
					desired_orientation = current_angle;
					//if(ACTIVATE_MOTION_CONTROLLER)
					//	start_motion_controller = 1;
					if(getselector()==1 || getselector()==3) {
						start_motion_controller = 1;
					} else {
						start_motion_controller = 0;
					}
					break;
               case 0:	// colors
					color_state++;
					if(color_state==5) {
						color_state=0;
					}

					if(color_state==0) {		// turn on blue
						pwm_blue = 0;
						pwm_green = 64;
						pwm_red = 64;					
					} else if(color_state==1) {	// turn on green
						pwm_blue = 64;
						pwm_green = 0;
						pwm_red = 64;
					} else if(color_state==2) {	// turn on red
						pwm_blue = 64;
						pwm_green = 64;
						pwm_red = 0;
					} else if(color_state==3) {	// turn on white
						pwm_blue = 0;
						pwm_green = 0;
						pwm_red = 0;
					} else if(color_state==4) {	// turn off
						pwm_blue = 64;
						pwm_green = 64;
						pwm_red = 64;
					}					
					SetDCOC3PWM_GB(pwm_blue,64);
					SetDCOC4PWM_GB(pwm_green,64);
					SetDCOC5PWM_GB(pwm_red,64);	

                  	break;
               default:
                  break;
            }

		}


		//READ FIFO_STATUS REGISTER
		toSend[0]=NRF_READ_REG + NRF_FIFO_STATUS;
		toSend[1]=0x00;
      	while(DataRdySPI1()) {
        	spi_data = ReadSPI1();
      	}	
		_LATC6 = 0;
		WriteSPI1(toSend[0]);
		while(!DataRdySPI1());
		spi_data = ReadSPI1();
		WriteSPI1(toSend[1]);
		while(!DataRdySPI1());
		fifo_status = ReadSPI1();
		_LATC6 = 1;

//		//READ STATUS REGISTER
//		toSend[0]=NRF_READ_REG + NRF_STATUS;
//		toSend[1]=0x00;
//      	while(DataRdySPI1()) {
//        	spi_data = ReadSPI1();
//      	}	
//		_LATC6 = 0;
//		WriteSPI1(toSend[0]);
//		while(!DataRdySPI1());
//		spi_data = ReadSPI1();
//		WriteSPI1(toSend[1]);
//		while(!DataRdySPI1());
//		status = ReadSPI1();
//		_LATC6 = 1;

		//printf("status:%x\r\n", status);
		//printf("fifo status:%x\r\n", fifo_status);
		//pritnf("data:");

		if(!(fifo_status&0x01)) {	// if RX FIFO not empty


//			// read rx payload width
//			toSend[0]=NRF_R_RX_PL_WID;
//			toSend[1]=0x00;
//	      	while(DataRdySPI1()) {
//	        	spi_data = ReadSPI1();
//	      	}	
//			_LATC6 = 0;
//			WriteSPI1(toSend[0]);
//			while(!DataRdySPI1());
//			spi_data = ReadSPI1();
//			WriteSPI1(toSend[1]);
//			while(!DataRdySPI1());
//			spi_data = ReadSPI1();
//			_LATC6 = 1;
//			printf("rx payload width:%x\r\n", spi_data);


			// read payload
			toSend[0]=NRF_RD_RX_PLOAD;
			toSend[1]=0x00;
      		while(DataRdySPI1()) {
        		spi_data = ReadSPI1();
      		}	
			_LATC6 = 0;
         	WriteSPI1(toSend[0]);
         	while(!DataRdySPI1());
         	spi_data = ReadSPI1();
			for(i=0; i<PAYLOAD_SIZE; i++) {
            	WriteSPI1(toSend[1]);
            	while(!DataRdySPI1());
            	data[i] = ReadSPI1();
				//printf("%x-", data[i]);
			}
			//printf("\r\n");
			_LATC6 = 1;	

			switch(data[0]) {
				case CHANGE_STATE:

					speedr = (data[29]&0x7F);
					speedl = (data[30]&0x7F);

					if((data[29]&0x80)==0x80) {	// motor right forward
						pwm_right_desired = MAX_PWM*speedr/100; // scale the speed received (0%..100%) to be in the range 0..MAX_PWM
					} else {				// backward
						pwm_right_desired = MAX_PWM*(-speedr)/100;
					}

					if((data[30]&0x80)==0x80) {	// motor left forward
						pwm_left_desired = MAX_PWM*speedl/100;
					} else {				// backward
						pwm_left_desired = MAX_PWM*(-speedl)/100;
					}

                	if (pwm_right_desired>MAX_PWM) pwm_right_desired=MAX_PWM;
                	if (pwm_left_desired>MAX_PWM) pwm_left_desired=MAX_PWM;
                	if (pwm_right_desired<-MAX_PWM) pwm_right_desired=-MAX_PWM;
                	if (pwm_left_desired<-MAX_PWM) pwm_left_desired=-MAX_PWM;

					if(getselector()==1 || getselector()==3) {
						start_motion_controller = 1;
					} else {
						start_motion_controller = 0;
					}
					if(pwm_right_desired==0 && pwm_left_desired==0) {
						start_motion_controller = 0;
					}					

					for(i=0; i<27; i++) {
						dataLED[i]=data[i+1]&0xFF;
					}
					pwm_red = 64-64*(dataLED[0]&0xFF)/100;
					pwm_blue = 64-64*(dataLED[8]&0xFF)/100;
					pwm_green = 64-64*(dataLED[16]&0xFF)/100;
					SetDCOC3PWM_GB(pwm_blue,64);
					SetDCOC4PWM_GB(pwm_green,64);
					SetDCOC5PWM_GB(pwm_red,64);
					

					if(data[28]== 1) {	// turn on IRs
						LED_IR1 = 0;
						LED_IR2 = 1;
					} else if(data[28]==2) {
						LED_IR1 = 1;
						LED_IR2 = 0;
					} else if(data[28]==3) {
						LED_IR1 = 0;
						LED_IR2 = 0;											
					} else {
						LED_IR1 = 1;
						LED_IR2 = 1;
					}

					break;

				case CHANGE_RF:
					init_rf_module(data[1]);					
					break;
			}


			//Flush RX Fifo
/*
			toSend[0]=FLUSH_RX;
			while(DataRdySPI1()) {
				spi_data = ReadSPI1();
			}
			_LATC6 = 0;;	
			WriteSPI1(toSend[0]);
			while(!DataRdySPI1());
			spi_data = ReadSPI1();
			_LATC6 = 1;				
*/

			// write ack payload
			toSend[0]= NRF_W_ACK_PAYLOAD_P0;
			toSend[1]=(unsigned char)(last_right_vel/10); //*100/1024); => overflow with vel > 650, e.g. 700*100 => 70000(!!!)/1024
			while(DataRdySPI1()) {
				spi_data = ReadSPI1();
			}
			_LATC6 = 0;
			WriteSPI1(toSend[0]);
			while(!DataRdySPI1());
			spi_data = ReadSPI1();
			WriteSPI1(toSend[1]);
			while(!DataRdySPI1());
			spi_data = ReadSPI1();
			_LATC6 = 1;
			while(DataRdySPI1()) {
		    	spi_data = ReadSPI1();
			}

		}

		//TMR1 = 0;
		//T1CONbits.TON = 1;					// start Timer1
		if(exec_motion_controller) {
			exec_motion_controller = 0;
			motion_controller();
		}
		//T1CONbits.TON = 0;
		//printf("controller takes %ld us\r\n", time_counter*32768 + TMR1/2);
		//time_counter = 0;		

		if(compute_left_vel) {
			last_left_vel = (unsigned int)(last_left_vel_sum/(long unsigned int)last_num_lvel_samples_avg);
			left_vel_changed = 1;
			compute_left_vel = 0;
		}

		if(compute_right_vel) {
			last_right_vel = (unsigned int)(last_right_vel_sum/(long unsigned int)last_num_rvel_samples_avg);
			right_vel_changed = 1;
			compute_right_vel = 0;
		}

		if(left_vel_changed && right_vel_changed) {		// last speed values updated for both motors
			left_vel_changed = 0;
			right_vel_changed = 0;
			speed_controller();
			debug_data_arr[0] = 0xFFFF;
			debug_data_arr[1] = (unsigned int) pwm_left_desired;
//			debug_data_arr[2] = last_left_vel;
			debug_data_arr[2] = (unsigned int) left_adc_to_duty;
			debug_data_arr[3] = (unsigned int) pwm_left_speed_controller;
//			debug_data_arr[5] = (unsigned int) pwm_left;			
			debug_data_arr[4] = (unsigned int) pwm_right_desired;
			debug_data_arr[5] = (unsigned int) right_adc_to_duty;
			debug_data_arr[6] = (unsigned int) pwm_right_speed_controller;
			fwrite(debug_data_arr, sizeof(unsigned int), 7, stdout);
		}

		if(exec_current_controller) {
			exec_current_controller = 0;
			current_controller();
		}
		
		if(update_pwm) {
			update_pwm = 0;
			while(TMR2 <= PERIOD_MOTORS/4 || TMR2 >= (PERIOD_MOTORS/4)*3);	// wait to be between 25% and 75% of the motors period before applying the update to the register
																			// to be sure the modifications are applied the next period
			pwm_left = pwm_left_current_controller;
			pwm_right = pwm_right_current_controller;
			//pwm_left = pwm_left_speed_controller;
			//pwm_right = pwm_right_speed_controller;

			if(pwm_left >= 0) {   // move forward
				OC1R = pwm_left;
			} else if(pwm_left < 0) {         // move backward
				OC1R = -pwm_left; //PERIOD_MOTORS+pwm_left;
			}
			if(pwm_right >= 0) {   // move forward
				OC2R = pwm_right;
			} else if(pwm_right < 0) {         // move backward
				OC2R = -pwm_right; //PERIOD_MOTORS+pwm_right;
			}
		}



	}

	return 0;
}


