//*********************************************************************************//
//                                                                                 //
//*********************************************************************************//

//=====================================================================
//	VARIABLES
//=====================================================================
#include <p24FJ64GA104.h>

//--Global--//
#define DEBUG_ASCII 0
#define DEBUG_MATLAB 0
#define PI 3.14159265
int i;
unsigned int init_pause_state = 0;	// used for the initial led sequence
unsigned long int time_counter = 0;
unsigned int debug_data_arr[10];
char left_vel_changed = 0;
char right_vel_changed = 0;

//--nRF--//
#define PAYLOAD_SIZE 31
#define CHANGE_STATE 0x0
#define CHANGE_RF 0x1
#define RF_ADDR 3004
unsigned char toSend[6];
unsigned char spi_data=-1;
unsigned char data[PAYLOAD_SIZE];
unsigned int dataLED[27];
unsigned char speedl=0, speedr=0;
unsigned char status = -1;
unsigned char fifo_status = -1;

//--IR Remote control--//
unsigned char ir_move = 0;

//--UART control--//
unsigned char DataAvailable;		// set when data are available on rx uart buffer
unsigned char count;
unsigned char Rxdata[4];

//--RGB LEDs--//
char color_state;
unsigned int pwm_red, pwm_green, pwm_blue;

//--IR LEDs--//
#define LED_IR1_DIR _TRISB12
#define LED_IR1 _LATB12
#define LED_IR2_DIR _TRISA10
#define LED_IR2 _LATA10

//--Motors--//
#define PERIOD_MOTORS 65000							// 30 Hz
#define STEP_MOTORS PERIOD_MOTORS/50				// 2% steps
#define MAX_PWM PERIOD_MOTORS/100*60				// 60% of maximum
signed long int pwm_right = 0;						// last pwm values before update
signed long int pwm_left = 0;
signed long int pwm_right_desired = 0;				// the pwm given by a command through uart, IR or RF
signed long int pwm_left_desired = 0;
char update_pwm = 0;
signed int speed_right_desired = 0;					// the speed in cm/s
signed int speed_left_desired = 0;

//--ADC--//
unsigned int battery_voltage = 0;
unsigned int ADCResult[8];

//--Accelerometer--//
char samples_received = 0;					// used to wait 50 samples for initial accelerometer calibration
char start_calibration = 0;					// start saving 50 samples for calibration
char calibration_done = 0;
unsigned int num_samples = 0;				// current number of samples received for calibration
unsigned int acc_samples[3][50];
unsigned int acc_x_offset = 0;				// values obtained during the calibration process; acc = raw_acc - offset
unsigned int acc_y_offset = 0;				// before calibration: values between -3g and +3g corresponds to values between 0 and 1024
unsigned int acc_z_offset = 0;				// after calibration: values between -3g and +3g corresponds to values between -512 and 512
unsigned int gravity = 0;
int acc_x = 0, acc_y = 0, acc_z = 0;		// raw accelerometer calibrated values
unsigned int abs_acc_x = 0, abs_acc_y = 0, abs_acc_z = 0;
signed int current_angle = 0;				// current orientation of the robot extracted from both the x and y axes
unsigned int num_samples_avg = 0;			// current number of samples received for filtering (average)
signed long int acc_x_sum = 0;				// sum of all the accelerometer values (at the end they will be divided by the total number of samples received)
signed long int acc_y_sum = 0;
signed long int acc_z_sum = 0;
signed int last_acc_x = 0;					// last accelerometer value filtered
signed int last_acc_y = 0;
signed int last_acc_z = 0;

//--Motion controller--//
#define NUM_SAMPLES 64						// number of samples used to filter the accelerometer values (average of the values)
#define ACTIVATE_MOTION_CONTROLLER 0
#define DELTA_ANGLE_FACT MAX_PWM/180	// scale the error to be in the range 0..PERIOD_MOTORS
#define P 8.0 	//0.3
#define D 1.0
#define I 0.0
signed int desired_orientation = 0;			// orientation saved when a command is received
signed int delta_angle = 0;					// difference between the current and desired orientations (represents the error for the controller)
signed int delta_angles[2] = {0};			// contains the previous and current errors (needed for the D parameter on a PID controller)
signed int delta_angles_sum = 0;			// sum of errors used for I parameter in the controller
char start_motion_controller = 0;			
char exec_motion_controller = 0;			// set at the beginning of the motors period
signed long int pwm_right_motion_controller = 0;	// the pwm values after the motion controller adaptation
signed long int pwm_left_motion_controller = 0;
#define wheel_distance 
signed int desired_angular_vel = 0;

//--Current consumption controller--//
#define CURRENT_LIM_UP 100					// used to change the limit of the current consumption for a certain pwm/duty: limit = (CURRENT_LIM_UP-duty)/2
#define MAX_CURRENT_MEAS 60					// max adc value measured when the motor is stopped
#define PWM_THRESHOLD PERIOD_MOTORS/100*20	// percentage of max velocity (motors period); values are between 0..PERIOD_MOTORS
unsigned int left_current_avg = 0;			// the left motor current consumption is sampled from 0 to duty cycle at each motor period; 
											// the average of all these samples is given as the new current consumption
unsigned int last_left_current = 0;			// current consumption of the previous/last period
unsigned int right_current_avg = 0;
unsigned int last_right_current = 0;
unsigned int pwm_current_lim = 0;			// current consumption upper limit for a certain pwm/duty: empirically is given by the formula: (CURRENT_LIM_UP-duty)/2
signed long int pwm_current_step = 0;		// current consumption controller step given by the formula: 0.75*(MAX_CURRENT_MEAS-pwm_curernt_lim)
char start_left_current_sampling = 0;		// set at the beginning of the motors period
char start_right_current_sampling = 0;
char exec_current_controller = 0;			// set at the end of motion controller because the values for the new pwm must be current limited by the controller just before update them
signed long int pwm_right_current_controller = 0;	// the pwm values after the current consumption controller adaptation
signed long int pwm_left_current_controller = 0;
signed long int pwm_right_working = 0;		// current pwm coming either from motion or speed controller
signed long int pwm_left_working = 0;

//--Speed controller--//
#define MAX_VEL_SAMPLES 64
#define NUM_SAMPLES_TO_DISCARD 0
unsigned int num_right_samples_discarded = 0;
unsigned int num_left_samples_discarded = 0;
unsigned int num_lvel_samples_avg = 0;		// current number of samples received for filtering (average) the left velocity value
unsigned int last_num_lvel_samples_avg = 0;
unsigned int num_rvel_samples_avg = 0;		// current number of samples received for filtering (average) the right velocity value
unsigned int last_num_rvel_samples_avg = 0;
unsigned long int left_vel_sum = 0;	// sum of all the adc values (at the end they will be divided by the total number of samples received)
unsigned long int last_left_vel_sum = 0;
unsigned long int right_vel_sum = 0;
unsigned long int last_right_vel_sum = 0;
unsigned int last_left_vel = 0;				// currnt velocity of the previous/last period
unsigned int last_right_vel = 0;
char start_left_vel_sampling = 0;			// set at the end of duty cycle
char start_right_vel_sampling = 0;
char compute_left_vel = 0;					// set at the end of the period (used to compute only once the average)
char compute_right_vel = 0;
unsigned int tmr2_value = 0;
unsigned int tmr5_value = 0;
unsigned int last_tmr2_value = 0;
unsigned int last_tmr5_value = 0;
signed long int pwm_right_speed_controller = 0;	// the pwm values after the speed controller adaptation
signed long int pwm_left_speed_controller = 0;
//signed int delta_left_speed = 0;
//signed int delta_right_speed = 0;
signed long int delta_left_speed = 0;
signed long int delta_right_speed = 0;
signed long int delta_left_speeds[2];
signed long int delta_right_speeds[2];
signed long int left_adc_to_duty = 0;
signed long int right_adc_to_duty = 0;
signed long int delta_left_speed_sum = 0;
signed long int delta_right_speed_sum = 0;
signed long int left_increment = 0;
signed long int right_increment = 0;
#define P_SPEED 0.5 //1.0	// 0.5
#define D_SPEED 0.5 //0.0	// 0.5
#define I_SPEED 0.5 //0.4	// 0.5
float p_speed_prop = 0.0;
float d_speed_prop = 0.0;
float i_speed_prop = 0.0;
